package com.clustercontrol.util;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * General Command Executor Class
 * @author takahatat
 */
public class CommandExecutor {

	private static Log log = LogFactory.getLog(CommandExecutor.class);

	// thread for command execution
	private final ExecutorService _commandExecutor;

	private final String[] _command;

	private final Charset _charset;
	public static final Charset _defaultCharset = Charset.forName("UTF-8");

	private final long _timeout;
	private static final long _defaultTimeout = 30000;
	public static final long _disableTimeout = -1;

	private final int _bufferSize;
	public static final int _defaultBufferSize = 8120;

	public static final Object _runtimeExecLock = new Object();

	public CommandExecutor(String[] command) {
		this(command, _defaultCharset);
	}

	public CommandExecutor(String[] command, Charset charset) {
		this(command, charset, _defaultTimeout);
	}

	public CommandExecutor(String[] command, long timeout) {
		this(command, _defaultCharset, timeout);
	}

	public CommandExecutor(String[] command, Charset charset, long timeout) {
		this(command, charset, timeout, _defaultBufferSize);
	}

	public CommandExecutor(String[] command, Charset charset, long timeout, int bufferSize) {
		this._command = command;
		this._charset = charset;
		this._timeout = timeout;
		this._bufferSize = bufferSize;

		if (log.isDebugEnabled()) log.debug("initializing " + this);

		if (_command == null) {
			throw new NullPointerException("command is not defined : " + this);
		}
		if (_charset == null) {
			throw new NullPointerException("charset is not defined : " + this);
		}

		_commandExecutor = Executors.newSingleThreadExecutor(
				new ThreadFactory() {
					private volatile int _count = 0;
					@Override
					public Thread newThread(Runnable r) {
						return new Thread(r, "CommandExecutor-" + _count++);
					}
				}
		);
	}
	@Override
	public String toString() {
		return this.getClass().getCanonicalName() + " [command = " + _command
		+ ", charset = " + _charset
		+ ", timeout = " + _timeout
		+ ", bufferSize = " + _bufferSize
		+ "]";
	}

	public CommandResult execute() {
		CommandExecuteTask task = new CommandExecuteTask(_command, _charset, _timeout, _bufferSize);
		Future<CommandResult> future = _commandExecutor.submit(task);
		if (log.isDebugEnabled()) log.debug("executing command. (command = " + task.commandLine + ", timeout = " + _timeout + " [msec]");

		// receive result
		CommandResult ret = null;
		try {
			if (_timeout == _disableTimeout) {
				ret = future.get();
			} else {
				ret = future.get(_timeout, TimeUnit.MILLISECONDS);
			}

			if (log.isDebugEnabled()) {
				log.debug("exit code : " + (ret != null ? ret.exitCode : null));
				log.debug("stdout : " + (ret != null ? ret.stdout : null));
				log.debug("stderr : " + (ret != null ? ret.stderr : null));
			}
		} catch (Exception e) {
			log.warn("command execution failure. (command = " + task.commandLine + ")", e);
		} finally {
			// release thread pool
			if (log.isDebugEnabled()) log.debug("releasing command threads.");
			_commandExecutor.shutdownNow();
		}

		return ret;
	}

	/**
	 * Command Result Class
	 * @author takahatat
	 */
	public class CommandResult {
		public final Integer exitCode;		// null when command timeout
		public final String stdout;		// null when command timeout
		public final String stderr;		// null when command timeout

		public CommandResult(int exitCode, String stdout, String stderr) {
			this.exitCode = exitCode;
			this.stdout = stdout;
			this.stderr = stderr;
		}
	}

	/**
	 * Command Execute Task
	 * @author takahatat
	 */
	public class CommandExecuteTask implements Callable<CommandResult> {

		// command string
		private final String[] command;
		private final String commandLine;

		// command timeout (receive timeout)
		private final long timeout;

		// maximun size of receive string
		public final int bufferSize;

		// charset of receive string
		public final Charset charset;

		// thread for receive stdout and stderr
		private final ExecutorService _receiverService;

		public CommandExecuteTask(String[] command, Charset charset, long timeout, int bufferSize) {
			this.command = command;
			this.charset = charset;
			this.timeout = timeout;
			this.bufferSize = bufferSize;

			String commandStr = "";
			for (String arg : command) {
				commandStr += commandStr == null ? arg : " " + arg;
			}
			commandLine = commandStr;
			if (log.isDebugEnabled()) log.debug("initializing " + this);

			_receiverService = Executors.newFixedThreadPool(
					2,
					new ThreadFactory() {
						private volatile int _count = 0;
						@Override
						public Thread newThread(Runnable r) {
							return new Thread(r, "CommendExecutor-" + _count++);
						}
					}
			);
		}

		@Override
		public String toString() {
			return this.getClass().getCanonicalName() + " [command = " + command
			+ ", charset = " + charset
			+ ", timeout = " + timeout
			+ ", bufferSize = " + bufferSize
			+ "]";
		}

		/**
		 * execute command
		 */
		@Override
		public CommandResult call() {
			Process process = null;

			Future<String> stdoutTask = null;
			Future<String> stderrTask = null;

			Integer exitCode = null;
			String stdout = null;
			String stderr = null;

			try {
				if (log.isDebugEnabled()) log.debug("starting child process : " + this);

				// workaround for JVM(Windows) Bug
				// Runtime#exec is not thread safe on Windows.
				synchronized (_runtimeExecLock) {
					process = Runtime.getRuntime().exec(command);
				}

				StreamReader stdoutReader = new StreamReader(process.getInputStream(), "CommandStdoutReader", _bufferSize);
				StreamReader stderrReader = new StreamReader(process.getErrorStream(), "CommandStderrReader", _bufferSize);
				stdoutTask = _receiverService.submit(stdoutReader);
				stderrTask = _receiverService.submit(stderrReader);

				if (log.isDebugEnabled()) log.debug("waiting child process : " + commandLine);
				exitCode = process.waitFor();
				if (log.isDebugEnabled()) log.debug("child process exited : " + commandLine);

				if (timeout == _disableTimeout) {
					stdout = stdoutTask.get();
					stderr = stderrTask.get();
				} else {
					stdout = stdoutTask.get(timeout, TimeUnit.MILLISECONDS);
					stderr = stderrTask.get(timeout, TimeUnit.MILLISECONDS);
				}
			} catch (Exception e) {
				log.warn("command executor failure. (command = " + commandLine + ")", e);
				stderr = "Internal Error : " + e.getMessage();
			} finally {
				if (log.isDebugEnabled()) log.debug("canceling stdout and stderr reader.");
				stdoutTask.cancel(true);
				stderrTask.cancel(true);
				if (process != null) {
					if (log.isDebugEnabled()) log.debug("destroying child process.");
					process.destroy();
				}
				// release thread pool
				if (log.isDebugEnabled()) log.debug("releasing receiver threads.");
				_receiverService.shutdownNow();
			}

			return new CommandResult(exitCode, stdout, stderr);
		}

		/**
		 * STDOUT and STDERR Reader Class<br/>
		 * @author takahatat
		 */
		public class StreamReader implements Callable<String> {

			// stream of stdout or stderr
			private final InputStream is;
			// receive buffer from stream
			private byte[] buf;

			private final String threadName;

			public StreamReader(InputStream is, String threadName, int bufferSize) {
				this.is = is;
				buf = new byte[bufferSize];
				this.threadName = threadName;

				if (log.isDebugEnabled()) log.debug("initializing " + this);
			}

			@Override
			public String toString() {
				return this.getClass().getCanonicalName() + " [threadName = " + threadName
				+ ", stream = " + is
				+ "]";
			}

			@Override
			public String call() {
				Thread.currentThread().setName(threadName);

				byte[] output = new byte[bufferSize];
				String outputStr = null;

				int size = 0;
				int used = 0;
				int total = 0;

				try {
					// read stream
					// until EOF (because some programs will be effected when stream is closed on running state)
					while ((size = is.read(buf, used, bufferSize - used)) != -1) {
						if (log.isTraceEnabled()) log.trace("reading " + size + "[byte] from a stream. (already received = " + total + "[byte])");

						total += size;
						if (total <= bufferSize) {
							// copy byte until buffer size
							if (log.isTraceEnabled()) log.debug("coping to output as stout/stderr. (offset = " + used + ", size = "  + size + "[byte])");
							System.arraycopy(buf, used, output, used, size);
						}

						if (total >= bufferSize) {
							// recycle buffer from beginning
							used = 0;
						} else {
							// offset position
							used += size;
						}
					}
					if (log.isDebugEnabled()) log.debug("reached end of stream.");
				} catch (IOException e) {
					log.warn("reading stream failure...", e);
				} finally {
					if (is != null) {
						try {
							if (log.isDebugEnabled()) log.debug("closing a stream.");
							is.close();
						} catch (IOException e) {
							log.warn("closing stream failure...", e);
						}
					}
				}

				// byte to string
				outputStr = total == 0 ? "" : new String(output, 0, total <= bufferSize ? total : bufferSize, charset);
				if (total > bufferSize) {
					log.warn("discarding command's output. (buffer = " + bufferSize + "[byte] < total = " + total + "[byte])");
				}

				return outputStr;
			}
		}
	}
}
